<?php
/**
* $Id: db2.php,v 1.1.2.3 2005/09/22 09:50:40 thorstenr Exp $
*
* db_db2
*
* The db_db2 class provides methods and functions for a IBM DB2 Version 8.2 
* database. This will only work with the PECL extension.
*
* @author       Thorsten Rinne <thorsten@phpmyfaq.de>
* @package      db_db2
* @since        2005-04-16
*
* Copyright:    (c) 2005 phpMyFAQ Team
*
* The contents of this file are subject to the Mozilla Public License
* Version 1.1 (the "License"); you may not use this file except in
* compliance with the License. You may obtain a copy of the License at
* http://www.mozilla.org/MPL/
*
* Software distributed under the License is distributed on an "AS IS"
* basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
* License for the specific language governing rights and limitations
* under the License.
*/

class db_db2
{
    /**
    * The connection object
    *
    * @var  mixed
    * @see  connect(), query(), dbclose()
    */
	var $conn = FALSE;
    
    /**
    * The query log string
    *
    * @var  string
    * @see  query()
    */
	var $sqllog = "";
    
    /**
    * The options array for DB2
    *
    * @var  array
    * @see  connect()
    */
    var $options = array('autocommit' => DB2_AUTOCOMMIT_ON);
	
    /**
     * Constructor
     *
     * @access  public
     * @author  Thorsten Rinne <thorsten@phpmyfaq.de>
     * @since   2005-04-16
     */
	function db_db2()
    {
    }
    
    function __construct()
    {
    }
    
    /**
    * Connects to the database.
    *
    * This function connects to a DB2 database
    *
    * @param   string $host
    * @param   string $username
    * @param   string $password
    * @param   string $db_name
    * @return  boolean TRUE, if connected, otherwise FALSE
    * @access  public
    * @author  Thorsten Rinne <thorsten@phpmyfaq.de>
    * @since   2005-04-16
    */
	function connect ($host, $user, $passwd, $db)
    {
		$this->conn = @db2_pconnect($db, $user, $passwd, $this->options);
		if (false == $this->conn) {
            print "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">\n";
            print "<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"en\" lang=\"en\">\n";
            print "<head>\n";
            print "    <title>phpMyFAQ Error</title>\n";
            print "    <meta http-equiv=\"content-type\" content=\"application/xhtml+xml; charset=utf-8\" />\n";
            print "</head>\n";
            print "<body>\n";
            print "<p align=\"center\">The connection to the DB2 server could not be established.</p>\n";
            print "<p align=\"center\">The error message of the DB2 server:<br />".db2_conn_errormsg()."</p>\n";
            print "</body>\n";
            print "</html>";
            return false;
        }
		return true;
    }
	
    /**
    * Sends a query to the database.
    *
    * This function sends a query to the database.
    *
    * @param   string $query
    * @return  mixed $result
    * @access  public
    * @author  Thorsten Rinne <thorsten@phpmyfaq.de>
    * @since   2005-04-16
    */
	function query($query)
    {
		$this->sqllog .= $query."<br />\n";
		return db2_exec($this->conn, $query);
    }
	
    /**
    * Escapes a string for use in a query
    *
    * @param   string
    * @return  string
    * @access  public
    * @author  Thorsten Rinne <thorsten@phpmyfaq.de>
    * @since   2005-04-16
    */
	function escape_string($string)
    {
      return addslashes($string);
    }
    
    /**
    * Fetch a result row as an object
    *
    * This function fetches a result row as an object.
    *
    * @param   mixed $result
    * @return  mixed
    * @access  public
    * @author  Thorsten Rinne <thorsten@phpmyfaq.de>
    * @since   2005-04-16
    */
    function fetch_object($result)
    {
        return db2_fetch_object($result);
   }
    
    
    /**
    * Number of rows in a result
    *
    * This function returns the number of rows in a result.
    *
    * @param   mixed $result
    * @return  mixed
    * @access  public
    * @author  Thorsten Rinne <thorsten@phpmyfaq.de>
    * @since   2005-04-16
    */
	function fetch_row($result)
    {
		return db2_fetch_row($result);
    }
	
    /**
    * Fetch a result row as an object
    *
    * This function fetches a result as an associative array.
    *
    * @param   mixed $result
    * @return  array
    * @access  public
    * @author  Thorsten Rinne <thorsten@phpmyfaq.de>
    * @since   2005-04-16
    */
	function fetch_assoc($result)
    {
		return db2_fetch_assoc($result);
    }
	
    /**
    * Number of rows in a result
    *
    * @param   mixed $result
    * @return  integer
    * @access  public
    * @author  Thorsten Rinne <thorsten@phpmyfaq.de>
    * @since   2005-04-16
    */
	function num_rows($result)
    {
		return db2_num_rows($result);
    }
	
    /**
    * Returns the ID of the latest insert
    *
    * @return  integer
    * @access  public
    * @author  Thorsten Rinne <thorsten@phpmyfaq.de>
    * @since   2005-04-16
    */
	function insert_id($table, $field)
    {
		$result = $this->query('SELECT max('.$field.') AS last_id FROM '.$table);
		return db2_result($result, 'last_id');
    }
    
    /**
    * Logs the queries
    *
    * @param   mixed $result
    * @return  integer
    * @access  public
    * @author  Thorsten Rinne <thorsten@phpmyfaq.de>
    * @since   2005-04-16
    */
	function sqllog()
    {
		return $this->sqllog;
    }
	
    
    /**
    * TODO: Implement this function
    */
	function version_check($target = "")
	{
		return false;
	}

	/**
    * Generates a result based on search a search string.
    *
    * This function generates a result set based on a search string.
    * FIXME: can extend to handle operands like google
    * @access  public
    * @author  Thorsten Rinne <thorsten@phpmyfaq.de>
    * @since   2005-04-16
    */
		$string = addslashes(trim($string));
		$fields = '';
        $join = '';
        $joined = '';
		$where = '';
        
		foreach ($assoc as $field) {
            if (empty($fields)) {
                $fields = $field;
			} else {
                $fields .= ", ".$field;
            }
		}
        
        if (isset($joinedTable) && $joinedTable != '') {
            $joined .= ' LEFT JOIN '.$joinedTable.' ON ';
        }
        
        if (is_array($joinAssoc)) {
            foreach ($joinAssoc as $joinedFields) {
                $join .= $joinedFields.' AND ';
                }
            $joined .= substr($join, 0, -4);
        }
        
        $keys = preg_split("/\s+/", $string);
        $numKeys = count($keys);
		$numMatch = count($match);
		
		for ($i = 0; $i < $numKeys; $i++) {
            if (strlen($where) != 0 ) {
                $where = $where;
            }
			$where = $where." (";
			for ($j = 0; $j < $numMatch; $j++) {
				if ($j != 0) {
				    $where = $where." OR ";
				}
		    	$where = $where.$match[$j]." LIKE '%".addslashes($keys[$i])."%'";
		    }
			
			$where .= ")";
		}
        
		foreach($cond as $field => $data) {
			if (empty($where)) {
				$where .= $field." = '".addslashes($data)."'";
            } else {
				$where .= " AND ".$field." = '".addslashes($data)."'";
            }
		}
        
        $query = "SELECT ".$fields." FROM ".$table.$joined." WHERE";
        
		if (!empty($where)) {
			$query .= " (".$where.")";
        }
        
        return $this->query($query);
	}
    
	/**
    * Returns the error string.
    *
    * This function returns the table status.
    *
    * @access  public
    * @author  Thorsten Rinne <thorsten@phpmyfaq.de>
    * @since   2005-04-16
    */
	function getTableStatus() 
	{
		$arr = array();
		$result = $this->query("SHOW TABLE STATUS");
		while ($row = $this->fetch_assoc($result)) {
            $arr[$row["Name"]] = $row["Rows"];
        }
		return $arr;
	}

	/**
	* Returns the next ID of a table
	*
	* @param   string      the name of the table
	* @param   string      the name of the ID column
	* @return  int
	* @access  public
	* @author  Thorsten Rinne <thorsten@phpmyfaq.de>
    * @since   2005-04-16
	*/
	function nextID($table, $id)
	{
	    $result = $this->query('SELECT max('.$id.') as current_id FROM '.$table);
	    $currentID = db2_result($result, 'current_id');
	    return ($currentID + 1);
	}
	
	/**
    * Returns the error string.
    *
    * This function returns the last error string.
    *
    * @access  public
    * @author  Thorsten Rinne <thorsten@phpmyfaq.de>
    * @since   2005-04-16
    */
	function error()
    {
        return db2_stmt_errormsg($this->conn);
    }

    /**
    * Returns the client version string.
    *
    * This function returns the version string.
    *
    * @access  public
    * @author  Thorsten Rinne <thorsten@phpmyfaq.de>
    * @since   2005-04-16
    */
	function client_version()
    {
        return db2_client_info();
    }

    /**
    * Returns the server version string.
    *
    * This function returns the version string.
    *
    * @access  public
    * @author  Thorsten Rinne <thorsten@phpmyfaq.de>
    * @since   2005-04-16
    */
	function server_version()
    {
        return db2_server_info();
    }

    /**
    * Closes the connection to the database.
    *
    * This function closes the connection to the database.
    *
    * @access  public
    * @author  Thorsten Rinne <thorsten@phpmyfaq.de>
    * @since   2005-04-16
    */
	function dbclose()
    {
		return db2_close($this->conn);
    }

}
?>
